/* beos_obuffer.cc

	BEOS output buffer written by John Fehr (jfehr@themall.net)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifdef BEOS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "args.h"
#include "obuffer.h"
#include "header.h"
#ifdef ULAW
#include "ulaw.h"
#endif

extern long write_to_buffer(char *buf,int length);

BEOSObuffer::BEOSObuffer (uint32 number_of_channels, MPEG_Args *maplay_args)
{
#ifdef DEBUG
  if (!number_of_channels || number_of_channels > MAXCHANNELS)
  {
    cerr << "BEOSObuffer: number of channels has to be in [1, " <<  MAXCHANNELS << "] !\n";
    exit (1);
  }
#endif

#ifdef ULAW
  if (number_of_channels > 1)
    cerr << "Are you sure you need stereo u-law output?\n";
#endif
  channels = number_of_channels;
  for (int i = 0; i < number_of_channels; ++i)
    bufferp[i] = buffer + i;
}

BEOSObuffer::~BEOSObuffer(void)
{
	return;
}

void BEOSObuffer::append (uint32 channel, int16 value)
{
  *bufferp[channel] = value;
  bufferp[channel] += channels;
}

#ifdef SEEK_STOP
void BEOSObuffer::clear_buffer(void)
{
}

void BEOSObuffer::set_stop_flag(void)
{
}
#endif // SEEK_STOP

void BEOSObuffer::write_buffer (int fd)
{
  int length = (int)((char *)bufferp[0] - (char *)buffer), writelength;

  if ((writelength = write_to_buffer ((char *)buffer, length)) != length)
  {
    // buffer has not been written completely
    if (writelength < 0)
    {
      perror ("write");
      exit (1);
    }
    length -= writelength;
    char *buffer_pos = (char *)buffer;
    do
    {
      buffer_pos += writelength;
      if ((writelength = write_to_buffer (buffer_pos, length)) < 0)
      {
	perror ("write");
	exit (1);
      }
    }
    while (length -= writelength);
  }

  for (int i = 0; i < channels; ++i)
    bufferp[i] = buffer + i;
}

Obuffer *create_obuffer(MPEG_Args *maplay_args)
{
	Obuffer *buffer;

	enum e_mode mode = maplay_args->MPEGheader->mode();
   enum e_channels which_channels = maplay_args->which_c;

	if (mode == single_channel || which_channels != both)
		buffer = new BEOSObuffer (1, maplay_args);	// mono
	else
		buffer = new BEOSObuffer (2, maplay_args);	// stereo

   return(buffer);
}

#endif // BEOS
